<?php
	
	/**
	 * ================================================================
	 * Dump variable number of arguments
	 * -
	 * Usage:
	 * echo xdump("hello", 123, ["a" => "b"], .......);
	 * ================================================================
	 *
	 * @param mixed ...$data Whatever data
	 *
	 * @return string
	 */
	function xdump(...$data) {
		static $counter = 1;
		
		# Different nice and decent colors for different file types
		static $colormap = ["integer" => "60afdc", "double" => "49c0b6", "float" => "49c0b6", "string" => "fd9f3e", "null" => "2c2c2c", "boolean" => "ff6c5f", "object" => "a26eea", "array" => "4257b2", "resource" => "d20962",];
		
		# If this is a CLI request, output will be plain, without HTML to also have a nicely formatted output in console
		static $cli = null;
		if ($cli === null) {
			$cli = (defined("STDIN") || array_key_exists("SHELL", $_ENV) || (defined("PHP_SAPI") && strtolower(PHP_SAPI) == "cli") || (function_exists("php_sapi_name") && strtolower((string)php_sapi_name()) == "cli"));
		}
		
		# Exception for trace string
		$exception   = new \Exception();
		$print_trace = htmlentities($exception->getTraceAsString(), ENT_QUOTES, "UTF-8");
		
		# Result output
		$output = "";
		
		# Iterate through passed data
		foreach ($data as $value) {
			$bool  = (bool)$value;
			$type  = strtolower(gettype($value));
			$color = (isset($colormap[$type]) ? $colormap[$type] : "f85a40");
			$color = ($value === true ? "47cf73" : ($value === false ? "ee4f4f" : $color));
			# For arrays, show values count, for strings show the length
			$type_info = (is_array($value) ? count($value) : (is_string($value) ? strlen($value) : ""));
			
			# CLI mode (for dumping values in console)
			if ($cli) {
				$output .= "\n============================ x dump ============================\n";
				$output .= "{$type}" . ($type_info !== "" ? "({$type_info})" : "") . " (~" . ($bool ? "true" : "false") . ") (call-no. {$counter})\n";
				if ($value !== null && $value !== true && $value !== false) {
					$output .= "----------------------------------------------------------------\n" . trim(print_r($value, true)) . "\n";
				}
				$output .= "----------------------------------------------------------------\n" . $print_trace . "\n================================================================\n";
			} else {
				# HTML output for web requests
				$type_info = ($type_info !== "" ? "<sup style='font-family:Calibri, Verdana, Helvetica, Arial, sans-serif !important;padding-left:2px;font-size:11px;'>({$type_info})</sup>" : "");
				$output    .= "<div class='xdump xdump-{$counter}' style='text-align:left;display:block !important;position:relative !important;clear:both !important;box-sizing:border-box;background-color:#{$color};border:4px solid rgba(255,255,255,0.5);border-radius:5px;padding:12px !important;margin:16px;font-family:Consolas,\"Courier New\", \"Arial\", sans-serif !important;font-size:13px;text-transform: none !important;line-height: initial !important;opacity:1.0 !important;float:none;'>";
				$output    .= "<div class='xdump-varinfo' style='box-sizing:border-box;background-color:rgba(255,255,255,0.4);border-radius:4px;padding:8px;margin:4px 0;color:rgba(0,0,0,0.72) !important;font-size:14px;'><span style='text-decoration: underline;font-weight:bold;font-family:Consolas,\"Courier New\", \"Arial\", sans-serif !important;color:rgba(0,0,0,0.72) !important;'>{$type}</span>{$type_info} <span style='opacity:0.5;color:rgba(0,0,0,0.72) !important;font-weight:normal !important;font-family:Consolas,\"Courier New\", \"Arial\", sans-serif !important;font-size:11px;'>(~" . ($bool ? "true" : "false") . ")</span></div>";
				if ($value !== null && $value !== true && $value !== false && $value !== []) {
					$output .= "<pre style='box-sizing:border-box;background-color:rgba(255,255,255,0.96);color:rgba(0,0,0,0.9);padding:8px;margin:8px 0;max-height:512px;overflow:auto;border:none;border-radius:4px;'>" . htmlentities(print_r($value, true), ENT_QUOTES, "UTF-8") . "</pre>";
				}
				$output .= "<div class='xdump-trace' style='font-family:Consolas,Monospace,\"Courier New\",Calibri, Verdana, Helvetica, Arial, sans-serif !important;text-align:left;box-sizing:border-box;padding:4px;color:rgba(255,255,255,0.4);font-size:10px;font-weight:normal;font-weight:100;'>" . nl2br($print_trace) . "</div></div>";
			}
			$counter++;
		}
		
		return $output;
	}
	
	/**
	 * Dump and die
	 *
	 * @param ...$data
	 *
	 * @return void
	 */
	function dd(...$data) {
		echo xdump(...$data);
		exit();
	}